@@ -1,6 +1,7 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ImplicitParams #-}
 {-# LANGUAGE NamedFieldPuns #-}
+{-# LANGUAGE RecordWildCards #-}
 {-# LANGUAGE ViewPatterns #-}
 
 module Test.DocTest
@@ -20,8 +21,10 @@ import           Prelude ()
 import           Prelude.Compat
 
 import qualified Data.Set as Set
+import           Data.List (intercalate)
 
 import           Control.Monad (unless)
+import           Control.Monad.Extra (ifM)
 import           System.Exit (exitFailure)
 import           System.IO
 import           System.Random (randomIO)
@@ -37,6 +40,7 @@ import GHC.Utils.Panic
 import Test.DocTest.Internal.Parse
 import Test.DocTest.Internal.Options
 import Test.DocTest.Internal.Runner
+import Test.DocTest.Internal.Nix (getNixGhciArgs)
 
 -- Cabal
 import Distribution.Simple
@@ -46,6 +50,9 @@ import Distribution.Simple
 import Test.DocTest.Helpers
   ( Library (libDefaultExtensions), extractCabalLibrary, findCabalPackage
   , libraryToGhciArgs )
+import Test.DocTest.Internal.Logging (LogLevel(..))
+
+import qualified Test.DocTest.Internal.Logging as Logging
 
 -- | Run doctest with given list of arguments.
 --
@@ -115,27 +122,39 @@ filterModules wantedMods0 allMods0
   nonExistingMods = Set.toList (wantedMods1 `Set.difference` allMods1)
   isSpecifiedMod Module{moduleName} = moduleName `Set.member` wantedMods1
 
-setSeed :: Bool -> ModuleConfig -> IO ModuleConfig
-setSeed quiet cfg@ModuleConfig{cfgRandomizeOrder=True, cfgSeed=Nothing} = do
-  -- Using an abslute number to prevent copy+paste errors
+setSeed :: (?verbosity :: LogLevel) => ModuleConfig -> IO ModuleConfig
+setSeed cfg@ModuleConfig{cfgRandomizeOrder=True, cfgSeed=Nothing} = do
+  -- Using an absolute number to prevent copy+paste errors
   seed <- abs <$> randomIO
-  unless quiet $
-    putStrLn ("Using freshly generated seed to randomize test order: " <> show seed)
+  Logging.log Info ("Using freshly generated seed to randomize test order: " <> show seed)
   pure cfg{cfgSeed=Just seed}
-setSeed _quiet cfg = pure cfg
+setSeed cfg = pure cfg
 
 -- | Run doctest for given library and config. Produce a summary of all tests.
 run :: Library -> Config -> IO Summary
 run lib Config{..} = do
+  nixGhciArgs <- ifM (pure cfgNix) getNixGhciArgs (pure [])
+
   let
     implicitPrelude = DisableExtension ImplicitPrelude `notElem` libDefaultExtensions lib
     (includeArgs, moduleArgs, otherGhciArgs) = libraryToGhciArgs lib
-    evalGhciArgs = otherGhciArgs ++ ["-XNoImplicitPrelude"]
+    evalGhciArgs = otherGhciArgs ++ ["-XNoImplicitPrelude"] ++ nixGhciArgs
+    parseGhciArgs = includeArgs ++ moduleArgs ++ otherGhciArgs ++ nixGhciArgs
 
-  modConfig <- setSeed cfgQuiet cfgModuleConfig
+  let
+    ?verbosity = cfgLogLevel
 
-  -- get examples from Haddock comments
-  allModules <- getDocTests (includeArgs ++ moduleArgs ++ otherGhciArgs)
-  runModules
-    modConfig cfgThreads cfgVerbose implicitPrelude evalGhciArgs
-    cfgQuiet (filterModules cfgModules allModules)
+  modConfig <- setSeed cfgModuleConfig
+
+  -- Get examples from Haddock comments
+  Logging.log Verbose "Parsing comments.."
+  Logging.log Debug ("Calling GHC API with: " <> unwords parseGhciArgs)
+  allModules <- getDocTests parseGhciArgs
+
+  -- Run tests
+  Logging.log Verbose "Running examples.."
+  let
+    filteredModules = filterModules cfgModules allModules
+    filteredModulesMsg = intercalate ", " (map moduleName filteredModules)
+  Logging.log Debug ("Running examples in modules: " <> filteredModulesMsg)
+  runModules modConfig cfgThreads implicitPrelude evalGhciArgs filteredModules
